'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.extractHandlers = extractHandlers;
exports.get = get;
exports.register = register;
exports.create = create;
exports.update = update;
exports.attachListeners = attachListeners;
exports.updateChildren = updateChildren;

var _reactLibInvariant = require('react/lib/invariant');

var _reactLibInvariant2 = _interopRequireDefault(_reactLibInvariant);

var assign = _Object$assign;

// Utilities

var handlerRE = /^on[A-Z]/g;

function extractHandlers(props) {
  var handlers = {};
  var rest = {};

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = _getIterator(_Object$keys(props)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var key = _step.value;

      if (key.match(handlerRE)) {
        handlers[key.slice(2, 3).toLowerCase() + key.slice(3)] = props[key];
      } else {
        rest[key] = props[key];
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator['return']) {
        _iterator['return']();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return { handlers: handlers, rest: rest };
}

// Definitions

var registry = {};

function get(type) {
  (0, _reactLibInvariant2['default'])(type in registry, 'No definition found for "' + type + '"');

  return registry[type];
}

var defaults = {
  factory: function factory(props) {
    return Ti.UI.createView(props);
  },

  create: function create(props, handlers, getChildren) {
    var view = this.factory(props);

    attachListeners(view, handlers);

    updateChildren(view, getChildren());

    return view;
  },

  update: function update(view, props, handlers) {
    // TODO: manage handlers

    for (var key in props) {
      var nextValue = props[key];

      if (key === 'value') {
        var oldValue = view[key];

        if (nextValue === oldValue) {
          continue;
        }
      }

      view[key] = nextValue;
    }

    // view.applyProperties(props);
  }
};

function register(shortName, apiName) {
  var config = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var definition = _extends({}, defaults, {
    shortName: shortName,
    apiName: apiName
  }, config);

  registry[shortName] = definition;
  registry[apiName] = definition;

  return definition;
}

function create(type, props, handlers, getChildren) {
  return get(type).create(props, handlers, getChildren);
}

function update(type, view, props, handlers) {
  return get(type).update(view, props, handlers);
}

function attachListeners(view, handlers) {
  for (var _name in handlers) {
    view.addEventListener(_name, handlers[_name]);
  }
}

function updateChildren(view, children) {
  view.removeAllChildren();

  // NOTE: Sloooooooow...

  var i = 0;
  var l = children.length;

  for (; i < l; ++i) {
    if (children[i]) {
      view.add(children[i]);
    }
  }
}

// Built-ins

register('ios-navigationwindow', 'Ti.UI.iOS.NavigationWindow', {
  factory: function factory(props) {
    return Ti.UI.iOS.createNavigationWindow(props);
  },

  create: function create(props, handlers, getChildren) {
    var children = getChildren();

    (0, _reactLibInvariant2['default'])(children.every(function (child) {
      return child.apiName === 'Ti.UI.Window';
    }), 'Only <window>s can be children of a <ios-navigationwindow>');

    var view = this.factory(_extends({}, props, {
      window: children[0]
    }));

    attachListeners(view, handlers);

    return view;
  }
});

register('tabgroup', 'Ti.UI.TabGroup', {
  factory: function factory(props) {
    return Ti.UI.createTabGroup(props);
  },

  create: function create(props, handlers, getChildren) {
    var children = getChildren();

    (0, _reactLibInvariant2['default'])(children.every(function (child) {
      return child.apiName === 'Ti.UI.Tab';
    }), 'Only <tab>s can be children of a <tabgroup>');

    var view = this.factory(_extends({}, props, {
      tabs: getChildren()
    }));

    attachListeners(view, handlers);

    return view;
  }
});

register('tab', 'Ti.UI.Tab', {
  factory: function factory(props) {
    return Ti.UI.createTab(props);
  },

  create: function create(props, handlers, getChildren) {
    var children = getChildren();

    (0, _reactLibInvariant2['default'])(children.every(function (child) {
      return child.apiName === 'Ti.UI.Window';
    }), 'Only <window>s can be children of a <tab>');

    var view = this.factory(_extends({}, props, {
      window: children[0]
    }));

    attachListeners(view, handlers);

    return view;
  }
});

register('window', 'Ti.UI.Window', {
  factory: function factory(props) {
    return Ti.UI.createWindow(props);
  }
});

register('view', 'Ti.UI.View', {
  factory: function factory(props) {
    return Ti.UI.createView(props);
  }
});

register('button', 'Ti.UI.Button', {
  factory: function factory(props) {
    return Ti.UI.createButton(props);
  }
});

register('input', 'Ti.UI.TextField', {
  factory: function factory(props) {
    return Ti.UI.createTextField(props);
  }
});

register('textarea', 'Ti.UI.TextArea', {
  factory: function factory(props) {
    return Ti.UI.createTextArea(props);
  }
});

register('label', 'Ti.UI.Label', {
  factory: function factory(props) {
    return Ti.UI.createLabel(props);
  }
});

register('list', 'Ti.UI.ListView', {
  factory: function factory(props) {
    return Ti.UI.createListView(props);
  }
});

register('switch', 'Ti.UI.Switch', {
  factory: function factory(props) {
    return Ti.UI.createSwitch(props);
  }
});

register('optiondialog', 'Ti.UI.OptionDialog', {
  factory: function factory(props) {
    return Ti.UI.OptionDialog(props);
  }
});

register('slider', 'Ti.UI.Slider', {
  factory: function factory(props) {
    return Ti.UI.Slider(props);
  }
});